@@ -38,6 +38,7 @@ typedef struct {
 	GPtrArray		*probs;
 	SoupSession		*session;
 	gboolean		 previous_para_was_short;
+	gchar			*previous_para_was_short_str;
 	guint			 para_chars_before_list;
 	guint			 number_paragraphs;
 } AsAppValidateHelper;
@@ -95,7 +96,7 @@ ai_app_validate_fullstop_ending (const gchar *tmp)
 			cnt++;
 	if (cnt++ > 1)
 		return FALSE;
-	str_len = strlen (tmp);
+	str_len = (guint) strlen (tmp);
 	if (str_len == 0)
 		return FALSE;
 	return tmp[str_len - 1] == '.';
@@ -179,7 +180,7 @@ as_app_validate_description_li (const gchar *text, AsAppValidateHelper *helper)
 		return;
 	}
 
-	str_len = strlen (text);
+	str_len = (guint) strlen (text);
 	if (str_len < length_li_min) {
 		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
@@ -240,11 +241,13 @@ as_app_validate_description_para (const gchar *text, AsAppValidateHelper *helper
 	}
 	helper->previous_para_was_short = FALSE;
 
-	str_len = strlen (text);
+	str_len = (guint) strlen (text);
 	if (str_len < length_para_min) {
 		/* we don't add the problem now, as we allow a short
 		 * paragraph as an introduction to a list */
 		helper->previous_para_was_short = TRUE;
+		g_free (helper->previous_para_was_short_str);
+		helper->previous_para_was_short_str = g_strdup (text);
 	}
 	if (str_len > length_para_max) {
 		ai_app_validate_add (helper,
@@ -303,7 +306,7 @@ as_app_validate_description_list (const gchar *text,
 	    helper->para_chars_before_list < (guint) length_para_before_list) {
 		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "Content before <ul> is too short [%d], at least %d characters required",
+				     "Content before <ul> is too short [%u], at least %u characters required",
 				     helper->para_chars_before_list,
 				     length_para_before_list);
 	}
@@ -375,17 +378,22 @@ as_app_validate_description (const gchar *xml,
 	if (helper->previous_para_was_short) {
 		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<p> is too short");
+				     "<p> is too short [%s]",
+				     helper->previous_para_was_short_str);
 	}
 	if (helper->number_paragraphs < number_para_min) {
 		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "Not enough <p> tags for a good description");
+				     "Not enough <p> tags for a good description [%u/%u]",
+				     helper->number_paragraphs,
+				     number_para_min);
 	}
 	if (helper->number_paragraphs > number_para_max) {
 		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "Too many <p> tags for a good description");
+				     "Too many <p> tags for a good description [%u/%u]",
+				     helper->number_paragraphs,
+				     number_para_max);
 	}
 	return TRUE;
 }
@@ -413,7 +421,7 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	gboolean require_correct_aspect_ratio = FALSE;
 	gdouble desired_aspect = 1.777777778;
 	gdouble screenshot_aspect;
-	gint status_code;
+	guint status_code;
 	guint screenshot_height;
 	guint screenshot_width;
 	guint ss_size_height_max = 900;
@@ -499,8 +507,8 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	}
 
 	/* check width matches */
-	screenshot_width = gdk_pixbuf_get_width (pixbuf);
-	screenshot_height = gdk_pixbuf_get_height (pixbuf);
+	screenshot_width = (guint) gdk_pixbuf_get_width (pixbuf);
+	screenshot_height = (guint) gdk_pixbuf_get_height (pixbuf);
 	if (as_image_get_width (im) != 0 &&
 	    as_image_get_width (im) != screenshot_width) {
 		ai_app_validate_add (helper,
@@ -585,7 +593,7 @@ as_app_validate_image (AsImage *im, AsAppValidateHelper *helper)
 
 	/* blank */
 	url = as_image_get_url (im);
-	if (strlen (url) == 0) {
+	if ((guint) strlen (url) == 0) {
 		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_VALUE_MISSING,
 				     "<screenshot> has no content");
@@ -638,16 +646,20 @@ as_app_validate_screenshot (AsScreenshot *ss, AsAppValidateHelper *helper)
 	}
 	tmp = as_screenshot_get_caption (ss, NULL);
 	if (tmp != NULL) {
-		str_len = strlen (tmp);
+		str_len = (guint) strlen (tmp);
 		if (str_len < length_caption_min) {
 			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<caption> is too short [%s]", tmp);
+					     "<caption> is too short [%s];"
+					     "shortest allowed is %u chars",
+					     tmp, length_caption_min);
 		}
 		if (str_len > length_caption_max) {
 			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<caption> is too long [%s]", tmp);
+					     "<caption> is too long [%s];"
+					     "longest allowed is %u chars",
+					     tmp, length_caption_max);
 		}
 		if (ai_app_validate_fullstop_ending (tmp)) {
 			ai_app_validate_add (helper,
@@ -796,7 +808,7 @@ as_app_validate_release (AsApp *app,
 {
 	const gchar *tmp;
 	guint64 timestamp;
-	guint number_para_max = 2;
+	guint number_para_max = 3;
 	guint number_para_min = 1;
 
 	/* relax the requirements a bit */
@@ -980,6 +992,18 @@ as_app_validate_is_content_license (const gchar *license)
 	return TRUE;
 }
 
+static void
+as_app_validate_helper_free (AsAppValidateHelper *helper)
+{
+	g_ptr_array_unref (helper->screenshot_urls);
+	g_free (helper->previous_para_was_short_str);
+	if (helper->session != NULL)
+		g_object_unref (helper->session);
+	g_free (helper);
+}
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(AsAppValidateHelper, as_app_validate_helper_free);
+
 /**
  * as_app_validate:
  * @app: a #AsApp instance.
@@ -996,11 +1020,9 @@ GPtrArray *
 as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 {
 	AsAppProblems problems;
-	AsAppValidateHelper helper;
 	GError *error_local = NULL;
 	GHashTable *urls;
 	GList *l;
-	GPtrArray *probs = NULL;
 	const gchar *description;
 	const gchar *id;
 	const gchar *key;
@@ -1030,6 +1052,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	guint number_para_min = 2;
 	guint str_len;
 	g_autoptr(GList) keys = NULL;
+	g_autoptr(AsAppValidateHelper) helper = g_new0 (AsAppValidateHelper, 1);
 
 	/* relax the requirements a bit */
 	if ((flags & AS_APP_VALIDATE_FLAG_RELAX) > 0) {
@@ -1074,19 +1097,12 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	}
 
 	/* set up networking */
-	helper.app = app;
-	helper.probs = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
-	helper.screenshot_urls = g_ptr_array_new_with_free_func (g_free);
-	helper.flags = flags;
-	helper.previous_para_was_short = FALSE;
-	helper.para_chars_before_list = 0;
-	helper.number_paragraphs = 0;
-	ret = as_app_validate_setup_networking (&helper, error);
-	if (!ret)
-		goto out;
-
-	/* success, enough */
-	probs = helper.probs;
+	helper->app = app;
+	helper->probs = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
+	helper->screenshot_urls = g_ptr_array_new_with_free_func (g_free);
+	helper->flags = flags;
+	if (!as_app_validate_setup_networking (helper, error))
+		return NULL;
 
 	/* id */
 	ret = FALSE;
@@ -1108,7 +1124,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 			ret = TRUE;
 		break;
 	case AS_APP_KIND_UNKNOWN:
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
 				     "<id> has invalid type attribute");
 
@@ -1117,16 +1133,13 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		if (g_str_has_suffix (id, ".firmware"))
 			ret = TRUE;
 		break;
-	case AS_APP_KIND_FONT:
-	case AS_APP_KIND_ADDON:
-	case AS_APP_KIND_SOURCE:
+	default:
 		/* anything goes */
 		ret = TRUE;
-	default:
 		break;
 	}
 	if (!ret) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_MARKUP_INVALID,
 				     "<id> does not have correct extension for kind");
 	}
@@ -1136,7 +1149,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	if (license != NULL) {
 		if (require_content_license &&
 		    !as_app_validate_is_content_license (license)) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TAG_INVALID,
 					     "<metadata_license> is not valid [%s]",
 					     license);
@@ -1146,7 +1159,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 				g_prefix_error (&error_local,
 						"<metadata_license> is not valid [%s]",
 						license);
-				ai_app_validate_add (&helper,
+				ai_app_validate_add (helper,
 						     AS_PROBLEM_KIND_TAG_INVALID,
 						     "%s", error_local->message);
 				g_clear_error (&error_local);
@@ -1157,7 +1170,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		switch (as_app_get_source_kind (app)) {
 		case AS_APP_SOURCE_KIND_APPDATA:
 		case AS_APP_SOURCE_KIND_METAINFO:
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TAG_MISSING,
 					     "<metadata_license> is not present");
 			break;
@@ -1174,7 +1187,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 			g_prefix_error (&error_local,
 					"<project_license> is not valid [%s]",
 					license);
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TAG_INVALID,
 					     "%s", error_local->message);
 			g_clear_error (&error_local);
@@ -1184,7 +1197,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		switch (as_app_get_source_kind (app)) {
 		case AS_APP_SOURCE_KIND_APPDATA:
 		case AS_APP_SOURCE_KIND_METAINFO:
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TAG_MISSING,
 					     "<project_license> is not present");
 			break;
@@ -1197,7 +1210,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	if (require_translation &&
 	    as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_APPDATA &&
 	    as_app_get_translations (app)->len == 0) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_TAG_MISSING,
 				     "<translation> not specified");
 	}
@@ -1205,7 +1218,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	/* pkgname */
 	if (as_app_get_pkgname_default (app) != NULL &&
 	    as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_METAINFO) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_TAG_INVALID,
 				     "<pkgname> not allowed in metainfo");
 	}
@@ -1213,7 +1226,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	/* appdata */
 	if (as_app_get_icon_default (app) != NULL &&
 	    as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_APPDATA) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_TAG_INVALID,
 				     "<icon> not allowed in appdata");
 	}
@@ -1222,7 +1235,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	if (as_app_get_extends(app)->len == 0 &&
 	    as_app_get_kind (app) == AS_APP_KIND_ADDON &&
 	    as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_METAINFO) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_TAG_MISSING,
 				     "<extends> is not present");
 	}
@@ -1231,12 +1244,12 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	update_contact = as_app_get_update_contact (app);
 	if (g_strcmp0 (update_contact,
 		       "someone_who_cares@upstream_project.org") == 0) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_TAG_INVALID,
 				     "<update_contact> is still set to a dummy value");
 	}
 	if (update_contact != NULL && strlen (update_contact) < 6) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
 				     "<update_contact> is too short [%s]",
 				     update_contact);
@@ -1245,7 +1258,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		switch (as_app_get_source_kind (app)) {
 		case AS_APP_SOURCE_KIND_APPDATA:
 		case AS_APP_SOURCE_KIND_METAINFO:
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TAG_MISSING,
 					     "<update_contact> is not present");
 			break;
@@ -1259,19 +1272,19 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	if (as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_APPDATA ||
 	    as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_METAINFO) {
 		if ((problems & AS_APP_PROBLEM_NO_XML_HEADER) > 0) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_MARKUP_INVALID,
 					     "<?xml> header not found");
 		}
 		if (require_copyright &&
 		    (problems & AS_APP_PROBLEM_NO_COPYRIGHT_INFO) > 0) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_VALUE_MISSING,
 					     "<!-- Copyright [year] [name] --> is not present");
 		}
 		if (deprectated_failure &&
 		    (problems & AS_APP_PROBLEM_UPDATECONTACT_FALLBACK) > 0) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TAG_INVALID,
 					     "<updatecontact> should be <update_contact>");
 		}
@@ -1279,7 +1292,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 
 	/* check invalid values */
 	if ((problems & AS_APP_PROBLEM_INVALID_PROJECT_GROUP) > 0) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_TAG_INVALID,
 				     "<project_group> is not valid");
 	}
@@ -1287,14 +1300,28 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	/* only allow XML in the specification */
 	if (require_appstream_spec_only &&
 	    (problems & AS_APP_PROBLEM_INVALID_XML_TAG) > 0) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_TAG_INVALID,
 				     "XML data contains unknown tag");
 	}
 
+	/* only allow XML in the specification */
+	if (problems & AS_APP_PROBLEM_EXPECTED_CHILDREN) {
+		ai_app_validate_add (helper,
+				     AS_PROBLEM_KIND_TAG_INVALID,
+				     "Expected children for tag");
+	}
+
+	/* only allow XML in the specification */
+	if (problems & AS_APP_PROBLEM_INVALID_KEYWORDS) {
+		ai_app_validate_add (helper,
+				     AS_PROBLEM_KIND_TAG_INVALID,
+				     "<keyword> invalid contents");
+	}
+
 	/* check for things that have to exist */
 	if (as_app_get_id (app) == NULL) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_TAG_MISSING,
 				     "<id> is not present");
 	}
@@ -1305,7 +1332,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	for (l = keys; l != NULL; l = l->next) {
 		key = l->data;
 		if (g_strcmp0 (key, "unknown") == 0) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TAG_INVALID,
 					     "<url> type invalid [%s]", key);
 		}
@@ -1314,7 +1341,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 			continue;
 		if (!g_str_has_prefix (tmp, "http://") &&
 		    !g_str_has_prefix (tmp, "https://")) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TAG_INVALID,
 					     "<url> does not start with 'http://' [%s]",
 					     tmp);
@@ -1322,51 +1349,50 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	}
 
 	/* screenshots */
-	as_app_validate_screenshots (app, &helper);
+	as_app_validate_screenshots (app, helper);
 
 	/* icons */
-	as_app_validate_icons (app, &helper);
+	as_app_validate_icons (app, helper);
 
 	/* releases */
-	ret = as_app_validate_releases (app, &helper, error);
-	if (!ret)
-		goto out;
+	if (!as_app_validate_releases (app, helper, error))
+		return NULL;
 
 	/* name */
 	name = as_app_get_name (app, "C");
 	if (name != NULL) {
-		str_len = strlen (name);
+		str_len = (guint) strlen (name);
 		if (str_len < length_name_min) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<name> is too short [%s]", name);
 		}
 		if (str_len > length_name_max) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<name> is too long [%s]", name);
 		}
 		if (ai_app_validate_fullstop_ending (name)) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<name> cannot end in '.' [%s]",
 					     name);
 		}
 		if (as_app_validate_has_hyperlink (name)) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<name> cannot contain a hyperlink [%s]",
 					     name);
 		}
 		if (require_sentence_case &&
-		    !as_app_validate_has_first_word_capital (&helper, name)) {
-			ai_app_validate_add (&helper,
+		    !as_app_validate_has_first_word_capital (helper, name)) {
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<name> requires sentence case [%s]",
 					     name);
 		}
 	} else if (require_name) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_TAG_MISSING,
 				     "<name> is not present");
 	}
@@ -1374,59 +1400,59 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	/* comment */
 	summary = as_app_get_comment (app, "C");
 	if (summary != NULL) {
-		str_len = strlen (summary);
+		str_len = (guint) strlen (summary);
 		if (str_len < length_summary_min) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<summary> is too short [%s]",
 					     summary);
 		}
 		if (str_len > length_summary_max) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<summary> is too long [%s]",
 					     summary);
 		}
 		if (ai_app_validate_fullstop_ending (summary)) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<summary> cannot end in '.' [%s]",
 					     summary);
 		}
 		if (as_app_validate_has_hyperlink (summary)) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<summary> cannot contain a hyperlink [%s]",
 					     summary);
 		}
 		if (require_sentence_case &&
-		    !as_app_validate_has_first_word_capital (&helper, summary)) {
-			ai_app_validate_add (&helper,
+		    !as_app_validate_has_first_word_capital (helper, summary)) {
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<summary> requires sentence case [%s]",
 					     summary);
 		}
 	} else if (require_name) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_TAG_MISSING,
 				     "<summary> is not present");
 	}
 	if (summary != NULL && name != NULL &&
 	    strlen (summary) < strlen (name)) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
 				     "<summary> is shorter than <name>");
 	}
 	description = as_app_get_description (app, "C");
 	if (description != NULL) {
 		ret = as_app_validate_description (description,
-						   &helper,
+						   helper,
 						   number_para_min,
 						   number_para_max,
 						   FALSE,
 						   &error_local);
 		if (!ret) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_MARKUP_INVALID,
 					     "%s", error_local->message);
 			g_error_free (error_local);
@@ -1436,21 +1462,21 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		if (name != NULL &&
 		    as_app_get_name_size (app) == 1 &&
 		    (problems & AS_APP_PROBLEM_INTLTOOL_NAME) == 0) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TRANSLATIONS_REQUIRED,
 					     "<name> has no translations");
 		}
 		if (summary != NULL &&
 		    as_app_get_comment_size (app) == 1 &&
 		    (problems & AS_APP_PROBLEM_INTLTOOL_SUMMARY) == 0) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TRANSLATIONS_REQUIRED,
 					     "<summary> has no translations");
 		}
 		if (description != NULL &&
 		    as_app_get_description_size (app) == 1 &&
 		    (problems & AS_APP_PROBLEM_INTLTOOL_DESCRIPTION) == 0) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TRANSLATIONS_REQUIRED,
 					     "<description> has no translations");
 		}
@@ -1459,27 +1485,27 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	/* developer_name */
 	name = as_app_get_developer_name (app, NULL);
 	if (name != NULL) {
-		str_len = strlen (name);
+		str_len = (guint) strlen (name);
 		if (str_len < length_name_min) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<developer_name> is too short [%s]",
 					     name);
 		}
 		if (str_len > length_name_max) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<developer_name> is too long [%s]",
 					     name);
 		}
 		if (as_app_validate_has_hyperlink (name)) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<developer_name> cannot contain a hyperlink [%s]",
 					     name);
 		}
 		if (as_app_validate_has_email (name)) {
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<developer_name> cannot contain an email address [%s]",
 					     name);
@@ -1488,13 +1514,13 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 
 	/* using deprecated names */
 	if (deprectated_failure && (problems & AS_APP_PROBLEM_DEPRECATED_LICENCE) > 0) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
 				     "<licence> is deprecated, use "
 				     "<metadata_license> instead");
 	}
 	if ((problems & AS_APP_PROBLEM_MULTIPLE_ENTRIES) > 0) {
-		ai_app_validate_add (&helper,
+		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_MARKUP_INVALID,
 				     "<application> used more than once");
 	}
@@ -1504,7 +1530,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		switch (as_app_get_source_kind (app)) {
 		case AS_APP_SOURCE_KIND_APPDATA:
 		case AS_APP_SOURCE_KIND_METAINFO:
-			ai_app_validate_add (&helper,
+			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TAG_MISSING,
 					     "<url> is not present");
 			break;
@@ -1512,9 +1538,5 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 			break;
 		}
 	}
-out:
-	g_ptr_array_unref (helper.screenshot_urls);
-	if (helper.session != NULL)
-		g_object_unref (helper.session);
-	return probs;
+	return helper->probs;
 }
