@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014-2015 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2014-2016 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -34,6 +34,8 @@
 #include "as-provide.h"
 #include "as-release.h"
 #include "as-screenshot.h"
+#include "as-review.h"
+#include "as-suggest.h"
 #include "as-content-rating.h"
 #include "as-translation.h"
 
@@ -77,7 +79,7 @@ typedef enum {
 	AS_APP_PARSE_FLAG_APPEND_DATA		= 8,	/* Since: 0.1.8 */
 	AS_APP_PARSE_FLAG_ALLOW_VETO		= 16,	/* Since: 0.2.5 */
 	AS_APP_PARSE_FLAG_USE_FALLBACKS		= 32,	/* Since: 0.4.1 */
-	AS_APP_PARSE_FLAG_ADD_ALL_METADATA	= 64,	/* Since: 0.5.18 */
+	AS_APP_PARSE_FLAG_ADD_ALL_METADATA	= 64,	/* Since: 0.6.1 */
 	/*< private >*/
 	AS_APP_PARSE_FLAG_LAST,
 } AsAppParseFlags;
@@ -87,19 +89,119 @@ typedef enum {
  * @AS_APP_SUBSUME_FLAG_NONE:		No special actions to use
  * @AS_APP_SUBSUME_FLAG_NO_OVERWRITE:	Do not overwrite already set properties
  * @AS_APP_SUBSUME_FLAG_BOTH_WAYS:	Copy unset properties both ways
- * @AS_APP_SUBSUME_FLAG_PARTIAL:	Only subsume a safe subset
+ * @AS_APP_SUBSUME_FLAG_PARTIAL:	Only subsume a safe subset (obsolete)
+ * @AS_APP_SUBSUME_FLAG_KIND:		Copy the kind
+ * @AS_APP_SUBSUME_FLAG_STATE:		Copy the state
+ * @AS_APP_SUBSUME_FLAG_BUNDLES:	Copy the bundles
+ * @AS_APP_SUBSUME_FLAG_TRANSLATIONS:	Copy the translations
+ * @AS_APP_SUBSUME_FLAG_RELEASES:	Copy the releases
+ * @AS_APP_SUBSUME_FLAG_KUDOS:		Copy the kudos
+ * @AS_APP_SUBSUME_FLAG_CATEGORIES:	Copy the categories
+ * @AS_APP_SUBSUME_FLAG_PERMISSIONS:	Copy the permissions
+ * @AS_APP_SUBSUME_FLAG_EXTENDS:	Copy the extends
+ * @AS_APP_SUBSUME_FLAG_COMPULSORY:	Copy the compulsory-for-desktop
+ * @AS_APP_SUBSUME_FLAG_SCREENSHOTS:	Copy the screenshots
+ * @AS_APP_SUBSUME_FLAG_REVIEWS:	Copy the reviews
+ * @AS_APP_SUBSUME_FLAG_CONTENT_RATINGS: Copy the content ratings
+ * @AS_APP_SUBSUME_FLAG_PROVIDES:	Copy the provides
+ * @AS_APP_SUBSUME_FLAG_ICONS:		Copy the icons
+ * @AS_APP_SUBSUME_FLAG_MIMETYPES:	Copy the mimetypes
+ * @AS_APP_SUBSUME_FLAG_VETOS:		Copy the vetos
+ * @AS_APP_SUBSUME_FLAG_LANGUAGES:	Copy the languages
+ * @AS_APP_SUBSUME_FLAG_NAME:		Copy the name
+ * @AS_APP_SUBSUME_FLAG_COMMENT:	Copy the comment
+ * @AS_APP_SUBSUME_FLAG_DEVELOPER_NAME	Copy the developer name
+ * @AS_APP_SUBSUME_FLAG_DESCRIPTION:	Copy the description
+ * @AS_APP_SUBSUME_FLAG_METADATA:	Copy the metadata
+ * @AS_APP_SUBSUME_FLAG_URL:		Copy the urls
+ * @AS_APP_SUBSUME_FLAG_KEYWORDS:	Copy the keywords
+ * @AS_APP_SUBSUME_FLAG_SOURCE_FILE:	Copy the source file
+ * @AS_APP_SUBSUME_FLAG_BRANCH:		Copy the branch
+ * @AS_APP_SUBSUME_FLAG_ORIGIN:		Copy the origin
+ * @AS_APP_SUBSUME_FLAG_METADATA_LICENSE: Copy the metadata license
+ * @AS_APP_SUBSUME_FLAG_PROJECT_LICENSE: Copy the project license
+ * @AS_APP_SUBSUME_FLAG_PROJECT_GROUP:	Copy the project group
+ * @AS_APP_SUBSUME_FLAG_SOURCE_KIND:	Copy the source kind
  *
  * The flags to use when subsuming applications.
  **/
 typedef enum {
-	AS_APP_SUBSUME_FLAG_NONE,
-	AS_APP_SUBSUME_FLAG_NO_OVERWRITE = 1,	/* Since: 0.1.4 */
-	AS_APP_SUBSUME_FLAG_BOTH_WAYS	 = 2,	/* Since: 0.1.4 */
-	AS_APP_SUBSUME_FLAG_PARTIAL	 = 4,	/* Since: 0.2.2 */
+	AS_APP_SUBSUME_FLAG_NONE		= 0,
+	AS_APP_SUBSUME_FLAG_NO_OVERWRITE	= 1lu << 0,	/* Since: 0.1.4 */
+	AS_APP_SUBSUME_FLAG_BOTH_WAYS		= 1lu << 1,	/* Since: 0.1.4 */
+	AS_APP_SUBSUME_FLAG_PARTIAL		= 1lu << 2,	/* Since: 0.2.2 */
+	AS_APP_SUBSUME_FLAG_KIND		= 1lu << 3,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_STATE		= 1lu << 4,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_BUNDLES		= 1lu << 5,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_TRANSLATIONS	= 1lu << 6,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_RELEASES		= 1lu << 7,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_KUDOS		= 1lu << 8,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_CATEGORIES		= 1lu << 9,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_PERMISSIONS		= 1lu << 10,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_EXTENDS		= 1lu << 11,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_COMPULSORY		= 1lu << 12,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_SCREENSHOTS		= 1lu << 13,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_REVIEWS		= 1lu << 14,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_CONTENT_RATINGS	= 1lu << 15,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_PROVIDES		= 1lu << 16,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_ICONS		= 1lu << 17,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_MIMETYPES		= 1lu << 18,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_VETOS		= 1lu << 19,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_LANGUAGES		= 1lu << 20,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_NAME		= 1lu << 21,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_COMMENT		= 1lu << 22,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_DEVELOPER_NAME	= 1lu << 23,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_DESCRIPTION		= 1lu << 24,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_METADATA		= 1lu << 25,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_URL			= 1lu << 26,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_KEYWORDS		= 1lu << 27,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_SOURCE_FILE		= 1lu << 28,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_BRANCH		= 1lu << 29,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_ORIGIN		= 1lu << 30,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_METADATA_LICENSE	= 1lu << 31,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_PROJECT_LICENSE	= 1lu << 32,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_PROJECT_GROUP	= 1lu << 33,	/* Since: 0.6.1 */
+	AS_APP_SUBSUME_FLAG_SOURCE_KIND		= 1lu << 34,	/* Since: 0.6.1 */
 	/*< private >*/
 	AS_APP_SUBSUME_FLAG_LAST,
 } AsAppSubsumeFlags;
 
+/* safe to do from a merge <component> */
+#define AS_APP_SUBSUME_FLAG_MERGE	(AS_APP_SUBSUME_FLAG_CATEGORIES | \
+					 AS_APP_SUBSUME_FLAG_COMMENT | \
+					 AS_APP_SUBSUME_FLAG_COMPULSORY | \
+					 AS_APP_SUBSUME_FLAG_CONTENT_RATINGS | \
+					 AS_APP_SUBSUME_FLAG_DESCRIPTION | \
+					 AS_APP_SUBSUME_FLAG_DEVELOPER_NAME | \
+					 AS_APP_SUBSUME_FLAG_EXTENDS | \
+					 AS_APP_SUBSUME_FLAG_ICONS | \
+					 AS_APP_SUBSUME_FLAG_KEYWORDS | \
+					 AS_APP_SUBSUME_FLAG_KUDOS | \
+					 AS_APP_SUBSUME_FLAG_LANGUAGES | \
+					 AS_APP_SUBSUME_FLAG_MIMETYPES | \
+					 AS_APP_SUBSUME_FLAG_NAME | \
+					 AS_APP_SUBSUME_FLAG_PERMISSIONS | \
+					 AS_APP_SUBSUME_FLAG_PROJECT_GROUP | \
+					 AS_APP_SUBSUME_FLAG_PROVIDES | \
+					 AS_APP_SUBSUME_FLAG_RELEASES | \
+					 AS_APP_SUBSUME_FLAG_REVIEWS | \
+					 AS_APP_SUBSUME_FLAG_SCREENSHOTS | \
+					 AS_APP_SUBSUME_FLAG_TRANSLATIONS | \
+					 AS_APP_SUBSUME_FLAG_URL)
+
+/* all properties */
+#define AS_APP_SUBSUME_FLAG_DEDUPE	(AS_APP_SUBSUME_FLAG_BRANCH | \
+					 AS_APP_SUBSUME_FLAG_BUNDLES | \
+					 AS_APP_SUBSUME_FLAG_KIND | \
+					 AS_APP_SUBSUME_FLAG_MERGE | \
+					 AS_APP_SUBSUME_FLAG_METADATA | \
+					 AS_APP_SUBSUME_FLAG_ORIGIN | \
+					 AS_APP_SUBSUME_FLAG_PROJECT_LICENSE | \
+					 AS_APP_SUBSUME_FLAG_SOURCE_FILE | \
+					 AS_APP_SUBSUME_FLAG_SOURCE_KIND | \
+					 AS_APP_SUBSUME_FLAG_STATE | \
+					 AS_APP_SUBSUME_FLAG_VETOS)
+
 /**
  * AsAppError:
  * @AS_APP_ERROR_FAILED:			Generic failure
@@ -208,6 +310,7 @@ typedef enum {
 	AS_APP_KIND_OS_UPGRADE,		/* Since: 0.5.10 */
 	AS_APP_KIND_SHELL_EXTENSION,	/* Since: 0.5.10 */
 	AS_APP_KIND_LOCALIZATION,	/* Since: 0.5.11 */
+	AS_APP_KIND_CONSOLE,		/* Since: 0.6.1 */
 	/*< private >*/
 	AS_APP_KIND_LAST
 } AsAppKind;
@@ -274,6 +377,40 @@ typedef enum {
 	AS_APP_STATE_LAST
 } AsAppState;
 
+/**
+ * AsAppScope:
+ * @AS_APP_SCOPE_UNKNOWN:			Unknown scope
+ * @AS_APP_SCOPE_USER:				User scope
+ * @AS_APP_SCOPE_SYSTEM:			System scope
+ *
+ * The application scope.
+ **/
+typedef enum {
+	AS_APP_SCOPE_UNKNOWN,				/* Since: 0.6.1 */
+	AS_APP_SCOPE_USER,				/* Since: 0.6.1 */
+	AS_APP_SCOPE_SYSTEM,				/* Since: 0.6.1 */
+	/*< private >*/
+	AS_APP_SCOPE_LAST
+} AsAppScope;
+
+/**
+ * AsAppMergeKind:
+ * @AS_APP_MERGE_KIND_UNKNOWN:			Unknown merge type
+ * @AS_APP_MERGE_KIND_NONE:			No merge to be done
+ * @AS_APP_MERGE_KIND_REPLACE:			Merge components, replacing
+ * @AS_APP_MERGE_KIND_APPEND:			Merge components, appending
+ *
+ * The component merge kind.
+ **/
+typedef enum {
+	AS_APP_MERGE_KIND_UNKNOWN,			/* Since: 0.6.1 */
+	AS_APP_MERGE_KIND_NONE,				/* Since: 0.6.1 */
+	AS_APP_MERGE_KIND_REPLACE,			/* Since: 0.6.1 */
+	AS_APP_MERGE_KIND_APPEND,			/* Since: 0.6.1 */
+	/*< private >*/
+	AS_APP_MERGE_KIND_LAST
+} AsAppMergeKind;
+
 #define	AS_APP_ERROR				as_app_error_quark ()
 
 AsApp		*as_app_new			(void);
@@ -284,10 +421,16 @@ const gchar	*as_app_source_kind_to_string	(AsAppSourceKind source_kind);
 const gchar	*as_app_state_to_string		(AsAppState	 state);
 const gchar	*as_app_kind_to_string		(AsAppKind	 kind);
 AsAppKind	 as_app_kind_from_string	(const gchar	*kind);
+AsAppScope	 as_app_scope_from_string	(const gchar	*scope);
+const gchar	*as_app_scope_to_string		(AsAppScope	 scope);
+AsAppMergeKind	 as_app_merge_kind_from_string	(const gchar	*merge_kind);
+const gchar	*as_app_merge_kind_to_string	(AsAppMergeKind	 merge_kind);
 
 /* getters */
 AsAppKind	 as_app_get_kind		(AsApp		*app);
 AsAppSourceKind	 as_app_get_source_kind		(AsApp		*app);
+AsAppScope	 as_app_get_scope		(AsApp		*app);
+AsAppMergeKind	 as_app_get_merge_kind		(AsApp		*app);
 AsAppState	 as_app_get_state		(AsApp		*app);
 AsAppTrustFlags	 as_app_get_trust_flags		(AsApp		*app);
 GList		*as_app_get_languages		(AsApp		*app);
@@ -305,10 +448,12 @@ GPtrArray	*as_app_get_architectures	(AsApp		*app);
 GPtrArray	*as_app_get_releases		(AsApp		*app);
 GPtrArray	*as_app_get_provides		(AsApp		*app);
 GPtrArray	*as_app_get_screenshots		(AsApp		*app);
+GPtrArray	*as_app_get_reviews		(AsApp		*app);
 GPtrArray	*as_app_get_content_ratings	(AsApp		*app);
 GPtrArray	*as_app_get_icons		(AsApp		*app);
 GPtrArray	*as_app_get_bundles		(AsApp		*app);
 GPtrArray	*as_app_get_translations	(AsApp		*app);
+GPtrArray	*as_app_get_suggests		(AsApp		*app);
 GHashTable	*as_app_get_names		(AsApp		*app);
 GHashTable	*as_app_get_comments		(AsApp		*app);
 GHashTable	*as_app_get_developer_names	(AsApp		*app);
@@ -320,6 +465,7 @@ const gchar	*as_app_get_icon_path		(AsApp		*app);
 const gchar	*as_app_get_id_filename		(AsApp		*app);
 const gchar	*as_app_get_id			(AsApp		*app);
 const gchar	*as_app_get_id_no_prefix	(AsApp		*app);
+const gchar	*as_app_get_unique_id		(AsApp		*app);
 const gchar	*as_app_get_pkgname_default	(AsApp		*app);
 const gchar	*as_app_get_source_pkgname	(AsApp		*app);
 const gchar	*as_app_get_origin		(AsApp		*app);
@@ -328,6 +474,7 @@ const gchar	*as_app_get_project_license	(AsApp		*app);
 const gchar	*as_app_get_metadata_license	(AsApp		*app);
 const gchar	*as_app_get_update_contact	(AsApp		*app);
 const gchar	*as_app_get_source_file		(AsApp		*app);
+const gchar	*as_app_get_branch		(AsApp		*app);
 const gchar	*as_app_get_name		(AsApp		*app,
 						 const gchar	*locale);
 const gchar	*as_app_get_comment		(AsApp		*app,
@@ -363,6 +510,10 @@ void		 as_app_set_kind		(AsApp		*app,
 						 AsAppKind	 kind);
 void		 as_app_set_source_kind		(AsApp		*app,
 						 AsAppSourceKind source_kind);
+void		 as_app_set_scope		(AsApp		*app,
+						 AsAppScope	 scope);
+void		 as_app_set_merge_kind		(AsApp		*app,
+						 AsAppMergeKind	 merge_kind);
 void		 as_app_set_state		(AsApp		*app,
 						 AsAppState	 state);
 void		 as_app_set_trust_flags		(AsApp		*app,
@@ -395,6 +546,8 @@ void		 as_app_set_description		(AsApp		*app,
 						 const gchar	*description);
 void		 as_app_set_source_file		(AsApp		*app,
 						 const gchar	*source_file);
+void		 as_app_set_branch		(AsApp		*app,
+						 const gchar	*branch);
 void		 as_app_set_priority		(AsApp		*app,
 						 gint		 priority);
 void		 as_app_add_category		(AsApp		*app,
@@ -420,6 +573,8 @@ void		 as_app_add_provide		(AsApp		*app,
 						 AsProvide	*provide);
 void		 as_app_add_screenshot		(AsApp		*app,
 						 AsScreenshot	*screenshot);
+void		 as_app_add_review		(AsApp		*app,
+						 AsReview	*review);
 void		 as_app_add_content_rating	(AsApp		*app,
 						 AsContentRating *content_rating);
 void		 as_app_add_icon		(AsApp		*app,
@@ -428,6 +583,8 @@ void		 as_app_add_bundle		(AsApp		*app,
 						 AsBundle	*bundle);
 void		 as_app_add_translation		(AsApp		*app,
 						 AsTranslation	*translation);
+void		 as_app_add_suggest		(AsApp		*app,
+						 AsSuggest	*suggest);
 void		 as_app_add_language		(AsApp		*app,
 						 gint		 percentage,
 						 const gchar	*locale);
@@ -488,6 +645,8 @@ AsRelease	*as_app_get_release_default	(AsApp		*app);
 gboolean	 as_app_convert_icons		(AsApp		*app,
 						 AsIconKind	 kind,
 						 GError		**error);
+gboolean	 as_app_equal			(AsApp		*app1,
+						 AsApp		*app2);
 
 /* deprecated */
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
