@@ -40,7 +40,7 @@
 #include <stdlib.h>
 #include <uuid.h>
 
-#include "as-app.h"
+#include "as-app-private.h"
 #include "as-enums.h"
 #include "as-node.h"
 #include "as-resources.h"
@@ -609,11 +609,11 @@ as_utils_license_to_spdx (const gchar *license)
 
 	/* go through the string looking for case-insensitive matches */
 	str = g_string_new ("");
-	license_len = strlen (license);
+	license_len = (guint) strlen (license);
 	for (i = 0; i < license_len; i++) {
 		gboolean found = FALSE;
 		for (j = 0; convert[j].old != NULL; j++) {
-			guint old_len = strlen (convert[j].old);
+			guint old_len = (guint) strlen (convert[j].old);
 			if (g_ascii_strncasecmp (license + i,
 						 convert[j].old,
 						 old_len) != 0)
@@ -811,13 +811,13 @@ as_pixbuf_sharpen (GdkPixbuf *src, gint radius, gdouble amount)
 		p_src_row = p_src;
 		p_blurred_row = p_blurred;
 		for (x = 0; x < width; x++) {
-			p_src_row[0] = interpolate_value (p_src_row[0],
+			p_src_row[0] = (guchar) interpolate_value (p_src_row[0],
 							  p_blurred_row[0],
 							  amount);
-			p_src_row[1] = interpolate_value (p_src_row[1],
+			p_src_row[1] = (guchar) interpolate_value (p_src_row[1],
 							  p_blurred_row[1],
 							  amount);
-			p_src_row[2] = interpolate_value (p_src_row[2],
+			p_src_row[2] = (guchar) interpolate_value (p_src_row[2],
 							  p_blurred_row[2],
 							  amount);
 			p_src_row += n_channels;
@@ -1490,7 +1490,7 @@ as_utils_guid_from_string (const gchar *str)
 	/* hash the namespace and then the string */
 	csum = g_checksum_new (G_CHECKSUM_SHA1);
 	g_checksum_update (csum, (guchar *) uu_namespace, 16);
-	g_checksum_update (csum, (guchar *) str, strlen(str));
+	g_checksum_update (csum, (guchar *) str, (gssize) strlen (str));
 	g_checksum_get_digest (csum, hash, &digestlen);
 
 	/* copy most parts of the hash 1:1 */
@@ -1520,12 +1520,12 @@ gchar *
 as_utils_version_from_uint32 (guint32 val, AsVersionParseFlag flags)
 {
 	if (flags & AS_VERSION_PARSE_FLAG_USE_TRIPLET) {
-		return g_strdup_printf ("%i.%i.%i",
+		return g_strdup_printf ("%u.%u.%u",
 					(val >> 24) & 0xff,
 					(val >> 16) & 0xff,
 					val & 0xffff);
 	}
-	return g_strdup_printf ("%i.%i.%i.%i",
+	return g_strdup_printf ("%u.%u.%u.%u",
 				(val >> 24) & 0xff,
 				(val >> 16) & 0xff,
 				(val >> 8) & 0xff,
@@ -1546,9 +1546,9 @@ as_utils_version_from_uint32 (guint32 val, AsVersionParseFlag flags)
 gchar *
 as_utils_version_from_uint16 (guint16 val, AsVersionParseFlag flags)
 {
-	return g_strdup_printf ("%i.%i",
-				(val >> 8) & 0xff,
-				val & 0xff);
+	return g_strdup_printf ("%u.%u",
+				(guint) (val >> 8) & 0xff,
+				(guint) val & 0xff);
 }
 
 /**
@@ -1606,7 +1606,7 @@ as_utils_version_parse (const gchar *version)
 		return g_strdup (version);
 	if (tmp == 0 || tmp < 0xff)
 		return g_strdup (version);
-	return as_utils_version_from_uint32 (tmp, AS_VERSION_PARSE_FLAG_USE_TRIPLET);
+	return as_utils_version_from_uint32 ((guint32) tmp, AS_VERSION_PARSE_FLAG_USE_TRIPLET);
 }
 
 /**
@@ -1638,8 +1638,8 @@ as_utils_string_replace (GString *string, const gchar *search, const gchar *repl
 	if (string->len == 0)
 		return 0;
 
-	search_len = strlen (search);
-	replace_len = strlen (replace);
+	search_len = (guint) strlen (search);
+	replace_len = (guint) strlen (replace);
 
 	do {
 		tmp = g_strstr_len (string->str + search_idx, -1, search);
@@ -1647,7 +1647,7 @@ as_utils_string_replace (GString *string, const gchar *search, const gchar *repl
 			break;
 
 		/* advance the counter in case @replace contains @search */
-		search_idx = tmp - string->str;
+		search_idx = (guint) (tmp - string->str);
 
 		/* reallocate the string if required */
 		if (search_len > replace_len) {
@@ -1679,7 +1679,7 @@ as_utils_string_replace (GString *string, const gchar *search, const gchar *repl
  *
  * Returns: a #GDateTime, or %NULL for error.
  *
- * Since: 0.5.18
+ * Since: 0.6.1
  **/
 GDateTime *
 as_utils_iso8601_to_datetime (const gchar *iso_date)
@@ -1704,5 +1704,151 @@ as_utils_iso8601_to_datetime (const gchar *iso_date)
 		return NULL;
 
 	/* create valid object */
-	return g_date_time_new_utc (dmy[0], dmy[1], dmy[2], 0, 0, 0);
+	return g_date_time_new_utc ((gint) dmy[0], (gint) dmy[1], (gint) dmy[2], 0, 0, 0);
+}
+
+static const gchar *
+_as_utils_fix_unique_id_part (const gchar *tmp)
+{
+	if (tmp == NULL || tmp[0] == '\0')
+		return AS_APP_UNIQUE_WILDCARD;
+	return tmp;
+}
+
+/**
+ * as_utils_unique_id_build:
+ * @scope: a #AsAppScope e.g. %AS_APP_SCOPE_SYSTEM
+ * @bundle_kind: System, e.g. 'package' or 'flatpak'
+ * @origin: Origin, e.g. 'fedora' or 'gnome-apps-nightly'
+ * @kind: #AsAppKind, e.g. %AS_APP_KIND_DESKTOP
+ * @id: AppStream ID, e.g. 'gimp.desktop'
+ * @branch: Branch, e.g. '3-20' or 'master'
+ *
+ * Builds a valid unique ID using available data.
+ *
+ * Returns: a unique name, or %NULL for error;
+ *
+ * Since: 0.6.1
+ */
+gchar *
+as_utils_unique_id_build (AsAppScope scope,
+			  AsBundleKind bundle_kind,
+			  const gchar *origin,
+			  AsAppKind kind,
+			  const gchar *id,
+			  const gchar *branch)
+{
+	const gchar *bundle_str = NULL;
+	const gchar *kind_str = NULL;
+	const gchar *scope_str = NULL;
+
+	g_return_val_if_fail (id != NULL, NULL);
+
+	if (kind != AS_APP_KIND_UNKNOWN)
+		kind_str = as_app_kind_to_string (kind);
+	if (scope != AS_APP_SCOPE_UNKNOWN)
+		scope_str = as_app_scope_to_string (scope);
+	if (bundle_kind != AS_BUNDLE_KIND_UNKNOWN)
+		bundle_str = as_bundle_kind_to_string (bundle_kind);
+	return g_strdup_printf ("%s/%s/%s/%s/%s/%s",
+				_as_utils_fix_unique_id_part (scope_str),
+				_as_utils_fix_unique_id_part (bundle_str),
+				_as_utils_fix_unique_id_part (origin),
+				_as_utils_fix_unique_id_part (kind_str),
+				_as_utils_fix_unique_id_part (id),
+				_as_utils_fix_unique_id_part (branch));
+}
+
+static inline guint
+as_utils_unique_id_find_part (const gchar *str)
+{
+	guint i;
+	for (i = 0; str[i] != '/' && str[i] != '\0'; i++);
+	return i;
+}
+
+/**
+ * as_utils_unique_id_valid:
+ * @unique_id: a unique ID
+ *
+ * Checks if a unique ID is valid i.e. has the correct number of
+ * sections.
+ *
+ * Returns: %TRUE if the ID is valid
+ *
+ * Since: 0.6.1
+ */
+gboolean
+as_utils_unique_id_valid (const gchar *unique_id)
+{
+	guint i;
+	guint sections = 1;
+	if (unique_id == NULL)
+		return FALSE;
+	for (i = 0; unique_id[i] != '\0'; i++) {
+		if (unique_id[i] == '/')
+			sections++;
+	}
+	return sections == AS_UTILS_UNIQUE_ID_PARTS;
+}
+
+static inline gboolean
+as_utils_unique_id_is_wildcard_part (const gchar *str, guint len)
+{
+	return len == 1 && str[0] == '*';
+}
+
+/**
+ * as_utils_unique_id_equal:
+ * @unique_id1: a unique ID
+ * @unique_id2: another unique ID
+ *
+ * Checks two unique IDs for equality allowing globs to match.
+ *
+ * Returns: %TRUE if the ID's should be considered equal.
+ *
+ * Since: 0.6.1
+ */
+gboolean
+as_utils_unique_id_equal (const gchar *unique_id1, const gchar *unique_id2)
+{
+	guint i;
+	guint last1 = 0;
+	guint last2 = 0;
+	guint len1;
+	guint len2;
+
+	/* trivial */
+	if (unique_id1 == unique_id2)
+		return TRUE;
+
+	/* invalid */
+	if (!as_utils_unique_id_valid (unique_id1) ||
+	    !as_utils_unique_id_valid (unique_id2))
+		return FALSE;
+
+	/* look at each part */
+	for (i = 0; i < AS_UTILS_UNIQUE_ID_PARTS; i++) {
+		const gchar *tmp1 = unique_id1 + last1;
+		const gchar *tmp2 = unique_id2 + last2;
+
+		/* find the slash or the end of the string */
+		len1 = as_utils_unique_id_find_part (tmp1);
+		len2 = as_utils_unique_id_find_part (tmp2);
+
+		/* either string was a wildcard */
+		if (!as_utils_unique_id_is_wildcard_part (tmp1, len1) &&
+		    !as_utils_unique_id_is_wildcard_part (tmp2, len2)) {
+			/* are substrings the same */
+			if (len1 != len2)
+				return FALSE;
+			if (memcmp (tmp1, tmp2, len1) != 0)
+				return FALSE;
+		}
+
+		/* advance to next section */
+		last1 += len1 + 1;
+		last2 += len2 + 1;
+	}
+	return TRUE;
 }
